import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, Component, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * CodeMirror component
 * Usage :
 * <codemirror [(ngModel)]="data" [config]="{...}"></codemirror>
 */
class CodemirrorComponent {
    constructor() {
        // tslint:disable-next-line:no-output-native
        this.change = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.focus = new EventEmitter();
        // tslint:disable-next-line:no-output-native
        this.blur = new EventEmitter();
        this.cursorActivity = new EventEmitter();
        this.val = '';
    }
    get value() {
        return this.val;
    }
    set value(v) {
        if (v !== this.val) {
            this.val = v;
            this.onChange(v);
        }
    }
    /**
     * On component view init
     */
    ngAfterViewInit() {
        this.config = this.config || {};
        this.codemirrorInit(this.config);
    }
    /**
     * Initialize codemirror
     */
    codemirrorInit(config) {
        if (!this.host) {
            throw new Error('Internal library error - host must be defined');
        }
        if (typeof window !== 'undefined') {
            import('codemirror')
                // tslint:disable-next-line:no-any
                .then((loaded) => {
                // in this case, the .default is required for Typescript Dynamic Imports
                const codemirror = loaded.default;
                this.instance = codemirror.fromTextArea(this.host?.nativeElement, config);
                this.instance.setValue(this.val);
                this.instance.on('change', () => {
                    this.updateValue(this.instance.getValue());
                });
                this.instance.on('focus', (instance, event) => {
                    this.focus.emit({ instance, event });
                });
                this.instance.on('cursorActivity', (instance) => {
                    this.cursorActivity.emit({ instance });
                });
                this.instance.on('blur', (instance, event) => {
                    this.blur.emit({ instance, event });
                });
            })
                .catch(err => {
                console.error('Cannot import codemirror: ', err);
            });
        }
    }
    /**
     * Value update process
     */
    updateValue(value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    }
    /**
     * Implements ControlValueAccessor
     */
    writeValue(value) {
        this.val = value || '';
        if (this.instance) {
            this.instance.setValue(this.val);
        }
    }
    onChange(_) { }
    onTouched() { }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
}
CodemirrorComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
CodemirrorComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.0", type: CodemirrorComponent, selector: "codemirror", inputs: { config: "config", value: "value" }, outputs: { change: "change", focus: "focus", blur: "blur", cursorActivity: "cursorActivity" }, providers: [
        {
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => CodemirrorComponent),
            multi: true
        }
    ], viewQueries: [{ propertyName: "host", first: true, predicate: ["host"], descendants: true }], ngImport: i0, template: `<textarea #host></textarea>`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'codemirror',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => CodemirrorComponent),
                            multi: true
                        }
                    ],
                    template: `<textarea #host></textarea>`
                }]
        }], propDecorators: { config: [{
                type: Input
            }], change: [{
                type: Output
            }], focus: [{
                type: Output
            }], blur: [{
                type: Output
            }], cursorActivity: [{
                type: Output
            }], host: [{
                type: ViewChild,
                args: ['host']
            }], value: [{
                type: Input
            }] } });

class CodemirrorModule {
}
CodemirrorModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CodemirrorModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorModule, declarations: [CodemirrorComponent], exports: [CodemirrorComponent] });
CodemirrorModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorModule, imports: [[]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.0", ngImport: i0, type: CodemirrorModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [CodemirrorComponent],
                    imports: [],
                    exports: [CodemirrorComponent]
                }]
        }] });

/*
 * Public API Surface of ngx-codemirror
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodemirrorComponent, CodemirrorModule };
